/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2012-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "env.h"
#include "core/log.h"


namespace Winix
{


namespace Fun
{

Env::Env()
{
	fun.url = L"env";
	puser   = 0;
	req_id  = 0;
}


bool Env::HasAccess()
{
	if( !cur->session->puser )
		return false;

	if( cur->request->IsParam(L"a") )
	{
		// show/change admin environment variables for a user

		if( !cur->session->puser->super_user )
			return false;
	}

	if( !GetUser() )
		return false;

return true;
}



bool Env::Parse(const std::wstring & env_str)
{
	space.clear();
	conf_parser.SetSpace(space);

return (conf_parser.ParseSpace(env_str) == PT::SpaceParser::ok);
}


bool Env::EditAdminEnv(long user_id, const std::wstring & env_str, bool use_ses_log)
{
	if( Parse(env_str) )
	{
		if( db->ChangeUserAdminEnv(user_id, space) == WINIX_ERR_OK )
		{
			User * puser = system->users.GetUser(user_id);

			if( puser )
				puser->aenv = space;

			return true;
		}
		else
		{
			log << log1 << "Evn: a database problem with changing environment variables for user: "
				<< cur->session->puser->name << ", id: " << cur->session->puser->id << logend;
		}
	}
	else
	{
		log << log2 << "Env: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}


bool Env::EditEnv(long user_id, const std::wstring & env_str, bool use_ses_log)
{
	if( Parse(env_str) )
	{
		if( db->ChangeUserEnv(user_id, space) == WINIX_ERR_OK )
		{
			User * puser = system->users.GetUser(user_id);

			if( puser )
				puser->env = space;

			return true;
		}
		else
		{
			log << log1 << "Evn: a database problem with changing admin environment variables for user: "
				<< cur->session->puser->name << ", id: " << cur->session->puser->id << logend;
		}
	}
	else
	{
		log << log2 << "Env: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}



void Env::SaveEnv()
{
	if( GetUser() )
	{
		const std::wstring & env_str = cur->request->PostVar(L"envvar");
		long user_id = GetUser()->id;
		bool status  = false;

		if( cur->request->IsParam(L"a") )
		{
			if( cur->session->puser->super_user )
				status = EditAdminEnv(user_id, env_str, true);
		}
		else
		{
			status = EditEnv(user_id, env_str, true);
		}

		if( status )
			system->RedirectToLastItem();
	}
}


User * Env::GetUser()
{
	if( cur->request->id != req_id )
	{
		req_id  = cur->request->id;
		puser   = 0;

		if( cur->session->puser )
		{
			if( cur->session->puser->super_user && cur->request->IsPostVar(L"userid") )
			{
				long id = Tol(cur->request->PostVar(L"userid"));
				puser   = system->users.GetUser(id);
			}
			else
			{
				puser = cur->session->puser;
			}
		}
	}

return puser;
}


void Env::MakePost()
{
	if( cur->session->puser )
	{
		if( cur->request->IsPostVar(L"changeuser") )
		{
			// show environments variables for the specified user
			if( GetUser() )
				log << log2 << "Env: changing user to: " << GetUser()->name << ", id: " << GetUser()->id << logend;
		}
		else
		{
			// save environment variables
			SaveEnv();
		}
	}
}


} // namespace


} // namespace Winix

