/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "templates.h"
#include "core/misc.h"

namespace Winix
{


namespace TemplatesFunctions
{
static std::string urlencode_tmp;
static std::string qencode_tmp;


void fil_urlencode(Info & i)
{
	UrlEncode(i.in.Str(), urlencode_tmp);
	i.out << R(urlencode_tmp);
}


void fil_qencode(Info & i)
{
	QEncode(i.in.Str(), qencode_tmp);
	i.out << R(qencode_tmp);
}


void fil_capitalize(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
		i.out << R(locale.ToCapital(str[a]));
}


void fil_tosmall(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
		i.out << R(locale.ToSmall(str[a]));
}


// a first letter in a sentence will be capitalized
void fil_firstup(Info & i)
{
	bool was_dot = true;
	const std::wstring & str = i.in.Str();
	
	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( was_dot )
		{
			if( str[a]!=' ' && str[a]!='\t' && str[a]!=13 && str[a]!=10 && str[a]!=160 )
				was_dot = false;

			i.out << R(locale.ToCapital(str[a]));
		}
		else
		{
			i.out << R(str[a]);
		}

		if( str[a] == '.' )
			was_dot = true;
	}
}


// a first letter in each word will be capitalized
void fil_first_wordup(Info & i)
{
	bool was_white = true;
	const std::wstring & str = i.in.Str();
	
	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( was_white )
		{
			i.out << R(locale.ToCapital(str[a]));
		}
		else
		{
			i.out << R(str[a]);
		}

		was_white = (str[a]==' ' || str[a]=='\t' || str[a]==13 || str[a]==10 || str[a]==160);
	}
}


bool fil_csv_has_colon_or_quote(const std::wstring & str)
{
	for(size_t i=0 ; i<str.size() ; ++i)
	{
		if( str[i] == ',' || str[i] == '"' )
			return true;
	}

return false;
}



void fil_csv_escape(Info & i)
{
	const std::wstring & str = i.in.Str();

	if( fil_csv_has_colon_or_quote(str) )
	{
		i.out << R("\"");

		for(size_t a=0 ; a<str.size() ; ++a)
		{
			if( str[a] == '"' )
				i.out << R("\"\"");
			else
				i.out << R(str[a]);
		}

		i.out << R("\"");
	}
	else
	{
		i.out << R(str);
	}
}



void fil_new_line_to_br(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( str[a] == '\n' )
			i.out << R("<br>\n");
		else
			i.out << R(str[a]);
	}
}



/*
 *  "  -> &#quot;
 *  '  -> &#39; (&apos; but IE8 has a problem with &apos;)
 */
void fil_html_quote(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( str[a] == '\"' )
			i.out << R("&quot;");
		else
		if( str[a] == '\'' )
			i.out << R("&#39;");
		else
			i.out << R(str[a]);
	}
}


/*
 *  10 -> &#10;
 *  13 -> &#13;
 */
void fil_html_newline(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( str[a] == 10 )
			i.out << R("&#10;");
		else
		if( str[a] == 13 )
			i.out << R("&#13;");
		else
			i.out << R(str[a]);
	}
}



} // namespace

} // namespace Winix

